<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Interface_directory\Admin\DriverRepositoryInterface;
use App\Models\Car;
use App\Models\Driver;
use App\Models\DriverCar;
use App\Models\DriverProfile;
use Carbon\Carbon;
use DateTime;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class DriverController extends Controller
{
    protected $driverRepository;
    public function __construct(DriverRepositoryInterface $driverRepository)
    {
        $this->driverRepository = $driverRepository;
    }

    public function index()
    {
        $all = $this->driverRepository->driver();
        $drivers = $all['drivers'];
        $countries = $all['countries'];
        $cars = $all['cars'];

        return view('admin.driver.index',compact('drivers','countries','cars'));
    }

    public function driversNotAccepted()
    {
        $all = $this->driverRepository->driversNotAccepted();
        $drivers = $all['drivers'];
        $countries = $all['countries'];
        $cars = $all['cars'];

        return view('admin.driver.index',compact('drivers','countries','cars'));
    }

    public function create()
    {
        //
    }

    public function store(Request $request)
    {
        $this->validate($request,[
            'name' => 'required',
            'email' => 'required|email',
            'phone' => 'required|numeric',
            'password' => 'required|min:6',
            'car_id' => 'required|exists:cars,id',
            'country_id' => 'required|exists:countries,id',
            'image' => 'required',
        ]);

        $driver = $this->driverRepository->storeDriver($request);

        if($driver == 'email_exist')
            return redirect(route('driver.index'))->with('error','Email already exists');
        elseif($driver == 'phone_exist')
            return redirect(route('driver.index'))->with('error','Phone already exists');
        else
            return redirect(route('driver.index'))->with('success','Driver added successfully');
    }

    public function show($id)
    {
        $driver = Driver::whereId($id)->first();
        $driver->count = json_decode($driver->count,true);
        $profile = DriverProfile::where('driver_id',$id)->first();
        return view('admin.driver.show',compact('driver','profile'));
    }

    public function changeStatus(Request $request)
    {
        $this->validate($request,[
            'driver_id' => 'required|exists:drivers,id'
        ]);

        $driver = $this->driverRepository->changStatus($request->driver_id);
        if($driver == 'suspend')
            return back()->with('success','Driver suspended successfully');
        else
            return back()->with('success','Driver activated successfully');
    }

    public function activeContract(Request $request)
    {
        $this->validate($request,[
            'driver_id' => 'required|exists:drivers,id'
        ]);

        $this->driverRepository->activeContract($request->driver_id);
        return back()->with('success','User activated successfully');
    }

    public function edit($id)
    {
        $driver = Driver::whereId($id)->first();
        $cars = Car::all();
        return view('admin.driver.single',compact('driver','cars'));
    }

    public function update(Request $request)
    {
        $this->validate($request,
            [
                'driver_id' => 'required|exists:drivers,id',
                'car_id' => 'sometimes',
                'name' => 'required',
                'email' => 'required|unique:drivers,email,'.$request->driver_id,
                'phone' => 'required|unique:drivers,phone,'.$request->driver_id,
                'image' => 'sometimes|image',
            ],
            [
                'name.required' => 'Name is required',
                'email.required' => 'Email is required',
                'phone.required' => 'Phone is required',
                'image.image' => 'Image is invalid',
            ]
        );


        $email_check = Driver::where('id','!=',$request->driver_id)->where('email', $request->email)->first();
        $phone_check = Driver::where('id','!=',$request->driver_id)->where('phone', $request->phone)->first();

        if($email_check) return back()->with('error', 'Sorry,email already exists,please change to another one');
        if($phone_check) return back()->with('error', 'Sorry,phone already exists,please change to another one');

        $this->driverRepository->updateDriver($request);
        return back()->with('success', 'Updated Successfully');

    }

    public function destroy($id)
    {
        //
    }

    public function search(Request $request)
    {
        $drivers = $this->driverRepository->search($request);
        return view('admin.driver.index',compact('drivers'));
    }

    public function acceptProfile(Request $request)
    {
        $driver_profile = DriverProfile::where('driver_id',$request->driver_id)->first();
        $driver = Driver::where('id',$request->driver_id)->first();
        $driver_car = DriverCar::where('driver_id',$request->driver_id)->first();

        if($driver_profile->name != null)
        {
            $driver->name = $driver_profile->name;
        }
        if($driver_profile->phone != null)
        {
            $phone_check = Driver::where('id','!=',$request->driver_id)->where('phone', $driver_profile->phone)->first();
            if($phone_check) return back()->with('error', 'Sorry,phone already exists,please change to another one');

            $driver->phone = $driver_profile->phone;
        }
        if($driver_profile->email != null)
        {
            $email_check = Driver::where('id','!=',$request->driver_id)->where('email', $driver_profile->email)->first();
            if($email_check) return back()->with('error', 'Sorry,email already exists,please change to another one');

            $driver->email = $driver_profile->email;
        }
        $driver->save();

        if($driver_profile->model != null)
        {
            $driver_car->model = $driver_profile->model;
        }
        if($driver_profile->color != null)
        {
            $driver_car->color = $driver_profile->color;
        }
        if($driver_profile->plat_number != null)
        {
            $driver_car->plat_number = $driver_profile->plat_number;
        }
        $driver_car->save();

        return back()->with('success', 'Updated Successfully');
    }

    public function collectFees(Request $request)
    {
        $driver = Driver::whereId($request->driver_id)->first();
        if (isset($driver)){
            $driver->app_fees = 0;
            $driver->reset_at = Carbon::now();
            $driver->save();
        }
        return back()->with('success', 'App Fees Collected Successfully');
    }
}
