<?php

namespace App\Http\Controllers\Admin;

use App\Models\AdminNotification;
use App\Models\Driver;
use App\Models\DriverNotify;
use App\Models\Notify;
use App\Models\User;
use App\Models\UserNotify;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class NotificationsController extends Controller
{
    public function index()
    {
        $notifications = AdminNotification::get();
        $data['notifications'] = $notifications;
        $data['title'] = 'رسائل النظام';
        return view('admin.notifications.index',compact('data'));
    }

    public function sendNotificationPage()
    {
        $data['title'] = trans('send_message');
        $users = User::get();
        $drivers = Driver::get();
        foreach ($users as $user){
            $user->type = 'user';
        }
        foreach ($drivers as $driver){
            $driver->type = 'driver';
        }
        $data['users'] = $users;
        $data['drivers'] = $drivers;
        return view('admin.notifications.send',compact('data'));
    }

    public function sendNotification(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'message' => 'required',
        ],[
            'message.required' => 'من فضلك أدخل الرد',
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator);
        }
        $user_type = explode('|', $request->person);
        if ($user_type['0'] == 'All_users'){
            $users_token = User::pluck('token')->toArray();
            $users = User::get();
            Notify::send($users_token,'message',$request->message,'رسالة من مدير النظام');
            foreach ($users as $user){
                $user_notification = new UserNotify();
                $user_notification->user_id = $user->id;
                $user_notification->save();
                $user_notification->translateOrNew('ar')->name = $request->message;
                $user_notification->translateOrNew('en')->name = $request->message;
                $user_notification->save();
            }
            AdminNotification::create([
                'message' => $request->message,
                'type' => 0  // All Users
            ]);
        }elseif ($user_type['0'] == 'All_drivers'){
            $drivers = Driver::get();
            $drivers_token = Driver::pluck('token')->toArray();
            Notify::send($drivers_token,'message',$request->message,'رسالة من مدير النظام');
            foreach ($drivers as $driver){
                $driver_notification = new DriverNotify();
                $driver_notification->driver_id = $driver->id;
                $driver_notification->save();
                $driver_notification->translateOrNew('ar')->name = $request->message;
                $driver_notification->translateOrNew('en')->name = $request->message;
                $driver_notification->save();
            }
            AdminNotification::create([
                'message' => $request->message,
                'type' => 1  // All Drivers
            ]);
        }else{
            if ($user_type['1'] == 'user'){
                $user = User::where('id',$user_type['0'])->first();
                Notify::send([$user->token],'message',$request->message,'رسالة من مدير النظام');
                $user_notification = new UserNotify();
                $user_notification->user_id = $user->id;
                $user_notification->save();
                $user_notification->translateOrNew('ar')->name = $request->message;
                $user_notification->translateOrNew('en')->name = $request->message;
                $user_notification->save();
                AdminNotification::create([
                    'user_id' => $user->id,
                    'message' => $request->message,
                    'type' => 2  // User
                ]);
            }elseif ($user_type['1'] == 'driver'){
                $driver = Driver::where('id',$user_type['0'])->first();
                Notify::send([$driver->token],'message',$request->message,'رسالة من مدير النظام');
                $driver_notification = new DriverNotify();
                $driver_notification->driver_id = $driver->id;
                $driver_notification->save();
                $driver_notification->translateOrNew('ar')->name = $request->message;
                $driver_notification->translateOrNew('en')->name = $request->message;
                $driver_notification->save();
                AdminNotification::create([
                    'driver_id' => $driver->id,
                    'message' => $request->message,
                    'type' => 3  // Driver
                ]);
            }
        }
        session()->flash('success', __("تم إرسال الرسالة بنجاح"));
        return redirect()->back();
//        }
//        else{
//            session()->flash('error', 'عذراً لا يمكنك الدخول لهذة الصفحة');
//            return redirect()->back();
//        }
    }

    public function delete(Request $request)
    {
//        if (Auth::guard('admin')->user()->can('delete_notifications')){
        $notification_id = $request->notification_id;
        $notification = AdminNotification::where('id',$notification_id)->first();
        if ($notification){
            $notification->delete();
            return redirect()->back()->with(['success' => 'تم الحذف بنجاح']);
        }else{
            return redirect()->back()->with(['error' => 'للأسف حدث خطأ ما']);
        }
//        }
//        else{
//            session()->flash('error', 'عذراً لا يمكنك الدخول لهذة الصفحة');
//            return redirect()->back();
//        }
    }

    public function destroyMulti(Request $request)
    {
//        if (Auth::guard('admin')->user()->can('delete_notifications')){
        // Init array of IDs
        $ids = $request->ids;
        foreach ($ids as $id) {
            $delete =$this->destroy($id);
            if (!$delete){
                return response()->json(['error' => 'للأسف حدث خطأ ما']);
            }
        }
        return response()->json(['success' => 'تم الحذف بنجاح']);
//        }
//        else{
//            session()->flash('error', 'عذراً لا يمكنك الدخول لهذة الصفحة');
//            return redirect()->back();
//        }
    }

    public function destroy($id)
    {
//        if (Auth::guard('admin')->user()->can('delete_notifications')){
        $notification_id = $id;
        $notification = AdminNotification::where('id',$notification_id)->first();
        return $notification->delete();
//        }
//        else{
//            session()->flash('error', 'عذراً لا يمكنك الدخول لهذة الصفحة');
//            return redirect()->back();
//        }
    }
}
