<?php

namespace App\Http\Controllers\Api\Driver;

use App\Http\Controllers\Interface_directory\Api\Driver\AuthRepositoryInterface;
use App\Http\Controllers\Interface_directory\Api\Driver\HomeRepositoryInterface;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{
    protected $authRepository;
    protected $homeRepository;
    public function __construct(AuthRepositoryInterface $authRepository, HomeRepositoryInterface $homeRepository)
    {
        $this->authRepository = $authRepository;
        $this->homeRepository = $homeRepository;
    }

    public function firstRegister(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'name' => 'required',
            'phone' => 'required|numeric',
            'email' => 'required',
            'password' => 'required',
            'code' => 'required|exists:countries,code',

        ]);
        
        //made by O2M
        $request->phone= $this->getPhone($request->code,$request->phone);
        
        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $register = $this->authRepository->firstRegister($request);
        if($register == 'phone_exist')
            return response()->json(msg($request,failed(),'phone_exist'));
        elseif($register == 'email_exist')
            return response()->json(msg($request,failed(),'email_exist'));
        else
            return response()->json(msgdata($request,success(),'success',$register));
    }

    public function secondRegister(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
            'model' => 'required',
            'color' => 'required',
            'plat_number' => 'required'
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $driver = $this->authRepository->secondRegister($request);
        return response()->json(msgdata($request,success(),'success',$driver));
    }

    public function codeSend(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'type' => 'required|in:activate,reset',
            'phone' => 'required',
            'code' => 'required|exists:countries,code',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $phone = $this->authRepository->sendSms($request->code,$request->phone,$request->type);
        if($phone == 'phone_not_exist')
            return response()->json(msg($request,failed(),'phone_not_exist'));
        else
            return response()->json(msgcode($request,success(),'success',$phone));
    }

    public function codeCheck(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'check_code' => 'required|numeric',
            'phone' => 'required',
            'code' => 'required|exists:countries,code',

        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $check = $this->authRepository->codeCheck($request);
        if($check == 'invalid_code') return response()->json(msg($request,failed(),'invalid_code'));
        elseif($check == 'activated_before') return response()->json(msg($request,failed(),'code_activated_before'));
        else
        return response()->json(msgdata($request,success(),'success',$check));
    }

    public function login(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'phone' => 'required',
            'password' => 'required',
            'token' => 'required',
            'device_token' => 'required',
            'code' => 'required|exists:countries,code',

        ]);

        //made by O2M
        $request->phone= $this->getPhone($request->code,$request->phone);
            
        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);


        if($request->code[strlen($request->code)-1] == $request->phone[0]){
            $request->phone = substr($request->phone,1,strlen($request->phone)-1);
        }
                // return response()->json(['status'=> 500, 'msg'=> $request->phone]);

        $login = $this->authRepository->login($request);
        if($login == 'not_active') return response()->json(msg($request,not_active(),'not_active'));
        elseif($login == 'waiting_admin') return response()->json(msg($request,failed(),'waiting_admin'));
        elseif($login == 'not_finished') return response()->json(msg($request,not_finished_register(),'not_finished_register'));
        elseif ($login == false) return response()->json(msg($request,failed(),'invalid_data'));
        elseif ($login == 'not_exist') return response()->json(msg($request,failed(),'phone_not_exist'));
        else
            $home = $this->homeRepository->home($request->token,$login->id);
            return response()->json(msgdata($request,success(),'success',$home));
    }

    public function forgetPassword(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
            'password' => 'required',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $this->authRepository->forgetPassword($request);
        return response()->json(msg($request,success(),'success'));
    }
    
    //made by O2M
    public function getPhone($code,$phone){
        if($code[strlen($code)-1] == $phone[0]){
            $phone = substr($phone,1,strlen($phone)-1);
        }
        return $phone;
    }
}
