<?php

namespace App\Http\Controllers\Api\Driver;

use App\Http\Controllers\Interface_directory\Api\Driver\AuthRepositoryInterface;
use App\Http\Controllers\Interface_directory\Api\Driver\DriverRepositoryInterface;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class DriverController extends Controller
{
    protected $driverRepository;
    protected $authRepository;
    public function __construct(DriverRepositoryInterface $driverRepository,AuthRepositoryInterface $authRepository)
    {
        $this->driverRepository = $driverRepository;
        $this->authRepository = $authRepository;
    }

    public function profile(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $profile = $this->driverRepository->profile($request);
            if($profile == 'password_changed')
                return response()->json(msg($request,success(),'password_changed'));
            elseif($profile == 'wrong_password')
                return response()->json(msg($request,failed(),'old_password'));

            else
                return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function credit(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $credit = $this->driverRepository->credit($request);
            return response()->json(msgdata($request,success(),'success',$credit));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function updatePayment(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
            'type' => 'required|in:1,2',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->driverRepository->updatePayment($request);
            return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function saveLocation(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->driverRepository->saveLocation($request);
            return response()->json(msg($request,success(),'location_saved'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function savedLocations(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $saved_locations = $this->driverRepository->savedLocations($request);
            return response()->json(msgdata($request,success(),'success',$saved_locations));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
}
