<?php

namespace App\Http\Controllers\Api\Driver;

use App\Http\Controllers\Interface_directory\Api\Driver\AuthRepositoryInterface;
use App\Http\Controllers\Interface_directory\Api\Driver\HomeRepositoryInterface;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class HomeController extends Controller
{
    protected $homeRepository;
    protected $authRepository;
    public function __construct(HomeRepositoryInterface $homeRepository,AuthRepositoryInterface $authRepository)
    {
        $this->homeRepository = $homeRepository;
        $this->authRepository = $authRepository;
    }

    public function home(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $home = $this->homeRepository->home($request->token,$request->driver_id);
            return response()->json(msgdata($request,success(),'success',$home));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function updateLocation(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
            'lat' => 'required',
            'lng' => 'required',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->homeRepository->updateLocation($request);
            return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
}
