<?php

namespace App\Http\Controllers\Api\Driver;

use App\Http\Controllers\Interface_directory\Api\Driver\AuthRepositoryInterface;
use App\Http\Controllers\Interface_directory\Api\Driver\TripRepositoryInterface;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class TripController extends Controller
{
    protected $tripRepository;
    protected $authRepository;
    public function __construct(TripRepositoryInterface $tripRepository,AuthRepositoryInterface $authRepository)
    {
        $this->tripRepository = $tripRepository;
        $this->authRepository = $authRepository;
    }

    public function acceptTrip(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
             $home = $this->tripRepository->acceptTrip($request);
            if($home == "false")
                return response()->json(msgdata($request,failed(),'invalid_data',$home));
            else
                return response()->json(msgdata($request,success(),'success',$home));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function cancelTrip(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
            'driver_id' => 'required|exists:drivers,id',
            'country_id' => 'required|exists:countries,id'
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $cancel = $this->tripRepository->cancelTrip($request,$request->header('lang'));
            if($cancel == "false")
                return response()->json(msg($request,failed(),'invalid_data'));
            else
                return response()->json(msg($request,success(),'canceled'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function cancelAfterAccept(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
            'driver_id' => 'required|exists:drivers,id',
            'country_id' => 'required|exists:countries,id'
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $cancel = $this->tripRepository->cancelAfterAccept($request,$request->header('lang'));
            if($cancel == false)
                return response()->json(msg($request,failed(),'invalid_data'));
            else
                return response()->json(msg($request,success(),'canceled'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function arrive(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $arrive = $this->tripRepository->arrive($request,$request->header('lang'));
            if($arrive =='cancelled'){
                return response()->json(msg($request,failed(),'already_canceled'));

            }
            if($arrive == "false") {
                return response()->json(msg($request, failed(), 'invalid_data'));
            }
            else {
                return response()->json(msg($request, success(), 'success'));
            }
            }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function updateAvailability(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->tripRepository->updateAvailability($request);
            return response()->json(msg($request,success(),'updated'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function updateGoHome(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->tripRepository->updateGoHome($request);
            return response()->json(msg($request,success(),'updated'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function tripHistory(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'driver_id' => 'required|exists:drivers,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $driver = $this->tripRepository->tripHistory($request);
            return response()->json(msgdata($request,success(),'success',$driver));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function calculateTrip(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->tripRepository->calculateTrip($request);
            return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function endTrip(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
            'country_id' => 'required|exists:countries,id',
            'address' => 'required',
            'lat' => 'required',
            'lng' => 'required',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $cost = $this->tripRepository->endTrip($request,$request->header('lang'));
            return response()->json(msgdata($request,success(),'success',$cost));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function collectMoney(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
            'money' => 'required|numeric',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $cost = $this->tripRepository->collectMoney($request,$request->header('lang'));
            return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
    public function startTrip(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->tripRepository->startTrip($request,$request->header('lang'));
            return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
    public function nextTrip(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->tripRepository->nextTrip($request,$request->header('lang'));
            return response()->json(msg($request,success(),'success'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
}
