<?php

namespace App\Http\Controllers\Api\User;

use App\Http\Controllers\Interface_directory\Api\User\AuthRepositoryInterface;
use App\Http\Controllers\Interface_directory\Api\User\TripRepositoryInterface;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class TripController extends Controller
{
    protected $tripRepository;
    protected $authRepository;
    public function __construct(TripRepositoryInterface $tripRepository,AuthRepositoryInterface $authRepository)
    {
        $this->tripRepository = $tripRepository;
        $this->authRepository = $authRepository;
    }

    public function createTrip(Request $request)
    {
        $validator  = Validator::make($request->all(),[
            'type' => 'required|in:urgent,scheduled',
            'car_id' => 'required|exists:cars,id',
            'user_id' => 'required|exists:users,id',
            'address' => 'required',
            'lat' => 'required',
            'lng' => 'required'
        ]);

        if($validator->fails())
        {
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);
        }

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
              $trip = $this->tripRepository->searchDriverCreateTrip($request,$request->header('lang'));
            if($trip == 'success_scheduled')
                return response()->json(msg($request,success(),'success_scheduled_trip'));
            elseif($trip == 'no_drivers')
                return response()->json(msg($request,failed(),'no_drivers'));
            else
                // return response()->json(msg($request,success(),'success'));
                return response()->json(msgdata($request,success(),'success',$trip));

        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function createScheduledTrip()
    {
        $trip = $this->tripRepository->scheduledTrip();

        if($trip == 'no_drivers')
            return 'no_drivers';
        elseif($trip == 'success')
            return 'success';
        else
            return 'false';
    }

    public function tripHistory(Request $request)
    {
        $validator  = Validator::make($request->all(),[
            'user_id' => 'required|exists:users,id',
        ]);

        if($validator->fails())
        {
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);
        }

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $trip_history = $this->tripRepository->tripHistory($request);
            return response()->json(msgdata($request,success(),'success',$trip_history));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function cancelTrip(Request $request)
    {
        $validator  = Validator::make($request->all(),[
            'trip_id' => 'required|exists:trips,id',
            'user_id' => 'required|exists:users,id',
        ]);

        if($validator->fails())
        {
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);
        }

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $this->tripRepository->cancelTrip($request,$request->header('lang'));
            return response()->json(msg($request,success(),'canceled'));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
}
