<?php

namespace App\Http\Controllers\Api\User;

use App\Http\Controllers\Interface_directory\Api\User\AuthRepositoryInterface;
use App\Http\Controllers\Interface_directory\Api\User\UserRepositoryInterface;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;

class UserController extends Controller
{
    protected $userRepository;
    protected $authRepository;
    public function __construct(UserRepositoryInterface $userRepository, AuthRepositoryInterface $authRepository)
    {
        $this->userRepository = $userRepository;
        $this->authRepository = $authRepository;
    }

    public function updateProfile(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'user_id' => 'required|exists:users,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $user = $this->userRepository->updateProfile($request);

        if($user == 'email_exist')
            return response()->json(msg($request,failed(),'email_exist'));
        elseif($user == 'phone_exist')
            return response()->json(msg($request,failed(),'phone_exist'));
        elseif($user == 'invalid_promo')
            return response()->json(msg($request,failed(),'invalid_promo'));
        elseif($user == 'promo_used')
            return response()->json(msg($request,failed(),'promo_used'));
        else
            return response()->json(msgdata($request,success(),'updated',$user));
    }

    public function updatePaymentType(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'user_id' => 'required|exists:users,id',
            'payment_type' => 'required|in:1,2',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        $user = $this->userRepository->updatePaymentType($request);
        return response()->json(msgdata($request,success(),'updated',$user));
    }

    public function showNotify(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'user_id' => 'required|exists:users,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $show_notify = $this->userRepository->showNotify($request,$request->header('lang'));
            return response()->json(msgdata($request,success(),'success',$show_notify));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }

    public function addPromo(Request $request)
    {
        $validator = Validator::make($request->all(),[
            'code' => 'required|numeric',
            'user_id' => 'required|exists:users,id',
        ]);

        if($validator->fails())
            return response()->json(['status'=> 401, 'msg'=> $validator->messages()]);

        if($this->authRepository->checkJWT($request->header('jwt')))
        {
            $promo = $this->userRepository->addPromo($request);
            if($promo == 'invalid_promo')
                return response()->json(msg($request,failed(),'invalid_promo'));
            elseif($promo == 'promo_used')
                return response()->json(msg($request,failed(),'promo_used'));
            else
                return response()->json(msgdata($request,success(),'success',$promo));
        }
        else return response()->json(msg($request, jwt(), 'invalid_data'));
    }
}
