<?php
/**
 * Created by PhpStorm.
 * User: Al Mohands
 * Date: 14/07/2019
 * Time: 11:53 ص
 */

namespace App\Http\Controllers\Eloquent\Admin;


use App\Http\Controllers\Interface_directory\Admin\DriverRepositoryInterface;
use App\Models\Car;
use App\Models\Country;
use App\Models\Driver;
use App\Models\DriverCar;
use App\Models\DriverCredit;
use App\Models\DriverProfile;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Input;
use Illuminate\Support\Str;

class DriverRepository implements DriverRepositoryInterface
{

    public function driver()
    {
        //$data['drivers'] = Driver::all();
        $data['drivers'] = Driver::orderBy('id','desc')->where('accept',1)->get();
        $data['countries'] = Country::all();
        $data['cars'] = Car::with('translation')->get();
        return $data;
    }

    public function driversNotAccepted()
    {
        $data['drivers'] = Driver::orderBy('id','desc')->where('accept',0)->get();
        $data['countries'] = Country::all();
        $data['cars'] = Car::with('translation')->get();
        return $data;
    }


    public function storeDriver($input)
    {
        $email = Driver::whereEmail($input->email)->select('id')->first();
        if($email)
            return 'email_exist';
        $phone = Driver::wherePhone($input->phone)->select('id')->first();
        if($phone)
            return 'phone_exist';

        $driver = Driver::create([
            'jwt' => Str::random(25),
            'active' => 1,
            'name' => $input->name,
            'email' => $input->email,
            'phone' => $input->phone,
            'password' => Hash::make($input->password),
            'car_id' => $input->car_id,
            'country_id' => $input->country_id,
            'finished' => 1,
            'image' => $input->image,
        ]);

        DriverCar::create([
            'driver_id' => $driver->id,
            'model' => $input->model,
            'color' => $input->color,
            'plat_number' => $input->plat_number,
            'image' => $input->image,
            'id_image' => $input->id_image,
            'driving_license_image' => $input->driving_license_image,
            'car_license_image' => $input->car_license_image,
        ]);

        DriverCredit::create([
            'driver_id' => $driver->id,
        ]);

        DriverProfile::create([
            'driver_id' => $driver->id,
        ]);
        return true;
    }

    public function changStatus($driver_id)
    {
        $worker = Driver::whereId($driver_id)->select('id','active')->first();

        if($worker->active == 1)
        {
            $worker->update(['active'=>0]);
            return 'suspend';
        }
        else{
            $worker->update(['active'=>1]);
            return 'active';
        }
    }

    public function activeContract($driver_id)
    {
        $worker = Driver::whereId($driver_id)->select('id','accept')->first();

        $worker->update(['accept'=>1]);
    }

    public function updateDriver($input)
    {
        $driver = Driver::where('id', $input->driver_id)->first();

        if($input->car_id) $driver->car_id = $input->car_id;
            $driver->name = $input->name;
            $driver->email = $input->email;
            $driver->phone = $input->phone;
        if($input->image)
        {
            $driver->image = $input->image;
        }

        $driver->save();
    }

    public function search($input)
    {
        $search = Input::get('search');
        return Driver::where(function($query) use($search) {
            $query->where('name','like','%'.$search.'%')
                ->orWhere('phone','like','%'.$search.'%')
                ->orWhere('email','like','%'.$search.'%')
                ->orWhereHas('driverCar',function ($q) use ($search){
                    $q->where('plat_number','like', '%'.$search.'%');
                });
        })->get();
    }
}
