<?php
/**
 * Created by PhpStorm.
 * User: Al Mohands
 * Date: 09/09/2019
 * Time: 02:24 م
 */

namespace App\Http\Controllers\Eloquent\Api\Driver;

use App\Models\Car;
use App\Models\Driver;
use App\Models\DriverCredit;
use App\Models\DriverNotify;
use App\Models\Message;
use App\Models\Notify;
use App\Models\Promo;
use App\Models\PromoUsed;
use App\Models\RushHourPeriod;
use App\Models\RushHourService;
use App\Models\ServiceCar;
use App\Models\Trip;
use App\Models\TripActiveRequest;
use App\Models\TripDistance;
use App\Models\TripPath;
use App\Models\TripStatus;
use App\Models\User;
use App\Models\UserNotify;
use Carbon\Carbon;

use App\Http\Controllers\Interface_directory\Api\Driver\TripRepositoryInterface;

class TripRepository implements TripRepositoryInterface
{
    public function tripActiveRequest($trip_id,$driver_id,$type)
    {
        if($type=='accept') {
            if (TripActiveRequest::where('trip_id', $trip_id)->where('driver_status', 1)->where('time_out', '>=', Carbon::now())->first()) {
                return null;
            }

            return TripActiveRequest::where('trip_id', $trip_id)->where('driver_id', $driver_id)
                ->where('driver_status', 0)->where('time_out', '>=', Carbon::now())->first();
        }
        else{
            return TripActiveRequest::where('trip_id', $trip_id)->where('driver_id', $driver_id)
                ->where('time_out', '>', Carbon::now())->first();

        }
    }

    public function acceptTrip($input)
    {
           $active_trip = $this->tripActiveRequest($input->trip_id,$input->driver_id,'accept');
        if($active_trip)
        {
            $user_id = Trip::where('id', $input->trip_id)->select('user_id')->first()->user_id;

            $active_trip->update(['driver_status' => 1]);

            Trip::where('id',$input->trip_id)->update(['driver_id'=>$input->driver_id]);
            $driver = Driver::where('id',$input->driver_id)->first();

            $driver->update([
                'status' => 1,
                'count_accepted_trips' => ($driver->count_accepted_trips + 1),
            ]);

            TripStatus::create([
                'trip_id' => $input->trip_id,
                'status_id' => 5
            ]);

            $ar_message = 'قام السائق بالموافقة و الرد علي طلبك';
            $en_message = 'Driver accept and responded to your request ';

//            UserNotify::create([
//                'user_id' => $user_id,
//                'trip_id' => $input->trip_id,
//                'en' => ['name' => $en_message],
//                'ar' => ['name' => $ar_message]
//            ]);
            $user_notification = new UserNotify();
            $user_notification->user_id = $user_id;
            $user_notification->trip_id = $input->trip_id;
            $user_notification->save();
            $user_notification->translateOrNew('ar')->name = $ar_message;
            $user_notification->translateOrNew('en')->name = $en_message;
            $user_notification->save();

            $user_token = User::where('id',$user_id)->pluck('token');

            Notify::send($user_token,'accept','accept','',$input->trip_id,$input->driver_id);

            $trip_path = Trip::where('id',$input->trip_id)->select('id','user_id','created_at')->with(['tripPath','user'])->first();
            return $trip_path;
        }
        else {return "false";}
    }

    public function cancelTrip($input,$lang)
    {
        $active_trip = $this->tripActiveRequest($input->trip_id,$input->driver_id,'cancel');

        if($active_trip)
        {
            $active_trip->update(['driver_status' => 2]);

            TripStatus::create([
                'trip_id' => $input->trip_id,
                'status_id' => 6
            ]);

            $driver = Driver::where('id',$input->driver_id)->select('id','count')->first();
            $driver_count = json_decode($driver->count,true);

            if($driver->count == null)
            {
                $driver->update([
                    'count' => collect(['2' => 1,'3' => 0])
                ]);

            }else{
                $driver->update([
                    'count' => collect(['2' => $driver_count['2']+1,'3' => 0])
                ]);

            }

            if($driver_count['2'] >= 3)
            {
                $driver_credit = DriverCredit::where('driver_id',$input->driver_id)->first();
                $service_fees = ServiceCar::where('country_id',$input->country_id)
                    ->select('id','cancel_before_fees')->first();
                $fees = 0;
                $fees+=$service_fees->cancel_before_fees;

                if($driver_credit)
                {
                    $driver_credit->update([
                        'admin_credit' => $fees
                    ]);
                }
                else{
                    DriverCredit::create([
                        'driver_id' => $input->driver_id,
                        'admin_credit' => $fees,
                    ]);
                }
            }
            $user_id = Trip::where('id', $input->trip_id)->select('user_id')->first()->user_id;

            $ar_message = 'قام السائق بالغاء الطلب.';
            $en_message = 'Driver cancel the order';

//            UserNotify::create([
//                'user_id' => $user_id,
//                'trip_id' => $input->trip_id,
//                'en' => ['name' => $en_message],
//                'ar' => ['name' => $ar_message]
//            ]);
            $user_notification = new UserNotify();
            $user_notification->user_id = $user_id;
            $user_notification->trip_id = $input->trip_id;
            $user_notification->save();
            $user_notification->translateOrNew('ar')->name = $ar_message;
            $user_notification->translateOrNew('en')->name = $en_message;
            $user_notification->save();

            $user_token = User::where('id',$user_id)->pluck('token');

            Notify::send($user_token,'reject','reject','',$input->trip_id,$input->driver_id);

            $active_trips = TripActiveRequest::where('trip_id',$input->trip_id)->where('driver_status',0)->get();

            if($active_trips)
            {
                foreach ($active_trips as $active_trip)
                {
                    $car_id = Trip::where('id',$active_trip->trip_id)->select('car_id','created_at')->first();
                    $trip_path = TripPath::where('trip_id',$active_trip->trip_id)->where('type','pick_up')->first();

                    $filter_trip = TripPath::filterbylatlng($trip_path->lat,$trip_path->lng,50,'drivers',$car_id);
                    if (count($filter_trip) > 0) {
                        $ar_message = 'لديك طلب رحلة جديد,الرجاء الإستجابة';
                        $en_message = 'You have a new trip request,please respond';

                        foreach ($filter_trip as $filter) {
                            DriverNotify::create([
                                'driver_id' => $filter->id,
                                'trip_id' => $active_trip->trip_id,
                                'en' => ['name' => $en_message],
                                'ar' => ['name' => $ar_message]
                            ]);

                            $driver_token = Driver::where('id', $filter->id)->pluck('token');
                            Notify::send($driver_token, 'trip', $lang == 'ar' ? $ar_message : $en_message, '', $active_trip->trip_id, $filter->id);
                        }
                    }
                }
            }

            return "true";
        }
        else return "false";
    }

    public function cancelAfterAccept($input,$lang)
    {
        $accept_trip = TripActiveRequest::where('trip_id',$input->trip_id)->where('driver_id', $input->driver_id)
            ->where('driver_status',1)->first();

        if($accept_trip)
        {
            $accept_trip->update(['driver_status' => 3]);
            Driver::where('id',$input->driver_id)->update(['status' => 0]);

            TripStatus::create([
                'trip_id' => $input->trip_id,
                'status_id' => 6
            ]);

            $driver = Driver::where('id',$input->driver_id)->select('id','count')->first();
            $driver_count = json_decode($driver->count,true);

            $driver->update([
                'count' => collect(['2'=>$driver_count['2'],'3' => $driver_count['3']+1]),
                'count_accepted_trips' => ($driver->count_accepted_trips - 1),
            ]);


            if($driver_count['3'] >= 3)
            {
                $driver_credit = DriverCredit::where('driver_id',$input->driver_id)->first();
                $service_fees = ServiceCar::where('country_id',$input->country_id)->select('id','cancel_after_fees')->first();
                $fees = 0;
                $fees+=$service_fees->cancel_before_fees;

                if($driver_credit)
                {
                    $driver_credit->update([
                        'admin_credit' => $fees
                    ]);
                }
            }
            $user_id = Trip::where('id', $input->trip_id)->select('user_id')->first()->user_id;

            $ar_message = 'قام السائق بالغاء الطلب.';
            $en_message = 'Driver cancel the order';

//            UserNotify::create([
//                'user_id' => $user_id,
//                'trip_id' => $input->trip_id,
//                'en' => ['name' => $en_message],
//                'ar' => ['name' => $ar_message]
//            ]);
            $user_notification = new UserNotify();
            $user_notification->user_id = $user_id;
            $user_notification->trip_id = $input->trip_id;
            $user_notification->save();
            $user_notification->translateOrNew('ar')->name = $ar_message;
            $user_notification->translateOrNew('en')->name = $en_message;
            $user_notification->save();

            $user_token = User::where('id',$user_id)->pluck('token');

            Notify::send($user_token,'reject','reject','',$input->trip_id,$input->driver_id);

            $active_trips = TripActiveRequest::where('trip_id',$input->trip_id)->where('driver_status',0)->get();

            if($active_trips)
            {
                foreach ($active_trips as $active_trip)
                {
                    $car_id = Trip::where('id',$active_trip->trip_id)->select('car_id')->first();
                    $trip_path = TripPath::where('trip_id',$active_trip->trip_id)->where('type','pick_up')->first();

                    $filter_trip = TripPath::filterbylatlng($trip_path->lat, $trip_path->lng, 50, 'drivers', $car_id->car_id);
                    if (count($filter_trip) > 0) {
                        $ar_message = 'لديك طلب رحلة جديد,الرجاء الإستجابة';
                        $en_message = 'You have a new trip request,please respond';

                        foreach ($filter_trip as $filter) {
                            DriverNotify::create([
                                'driver_id' => $filter->id,
                                'trip_id' => $active_trip->trip_id,
                                'en' => ['name' => $en_message],
                                'ar' => ['name' => $ar_message]
                            ]);

                            $driver_token = Driver::where('id', $filter->id)->pluck('token');
                            Notify::send($driver_token, 'trip', $lang == 'ar' ? $ar_message : $en_message, '', $active_trip->trip_id, $filter->id);
                        }
                    }
                }
            }

            return true;
        }
        else return false;
    }

    public function arrive($input,$lang)
    {
        $active_trip = TripActiveRequest::where('trip_id',$input->trip_id)->where('driver_id', $input->driver_id)->where('driver_status',1)->first();

        if($active_trip)
        {
            $user_id = Trip::where('id', $input->trip_id)->select('user_id')->first()->user_id;
             $trip=Trip::where('id', $input->trip_id)->first();

            if($trip->order_action==2){
                $user_token = User::where('id',$user_id)->pluck('token');
                $ar_message = 'قام المستخدم بالغاء طلبك';
                $en_message = 'User Cancelled ';

                Notify::send($user_token, 'time_out', $lang == 'ar' ? $ar_message : $en_message, '', $active_trip->trip_id, $input->driver_id);
                return 'cancelled';

            }

            $active_trip->update(['arrive' => 1]);

            TripStatus::create([
                'trip_id' => $input->trip_id,
                'status_id' => 7
            ]);
            $ar_message = 'قام السائق بالوصول.';
            $en_message = 'Driver arrive.';

//            UserNotify::create([
//                'user_id' => $user_id,
//                'trip_id' => $input->trip_id,
//                'en' => ['name' => $en_message],
//                'ar' => ['name' => $ar_message]
//            ]);
            $user_notification = new UserNotify();
            $user_notification->user_id = $user_id;
            $user_notification->trip_id = $input->trip_id;
            $user_notification->save();
            $user_notification->translateOrNew('ar')->name = $ar_message;
            $user_notification->translateOrNew('en')->name = $en_message;
            $user_notification->save();

            $user_token = User::where('id',$user_id)->pluck('token');
            Notify::send($user_token, 'arrive', $lang == 'ar' ? $ar_message : $en_message, '', $active_trip->trip_id, $input->driver_id);
            return 'true';
        }else return 'false';
    }

    public function updateAvailability($input)
    {
        $driver = Driver::where('id',$input->driver_id)->first();
        $driver->available = $input->available;
        $driver->lat = $input->lat;
        $driver->lng = $input->lng;
        $driver->save();
    }

    public function updateGoHome($input)
    {
        $driver = Driver::where('id',$input->driver_id)->first();
        $driver->go_home = $input->go_home;
        $driver->destination_lat = $input->destination_lat;
        $driver->destination_lng = $input->destination_lng;
        $driver->save();
    }

    public function tripHistory($input)
    {
        $trips = Trip::where('driver_id',$input->driver_id)->select('id','rate','order_action','total','created_at')->get();
        foreach ($trips as $trip)
        {
            $trip['rate'] = $trip->rate['driver'];
            $pick_up = TripPath::where('trip_id',$trip->id)->where('type','pick_up')->select('address')->first();
            $where_to = TripPath::where('trip_id',$trip->id)->where('type','where_to')->select('address')->first();
            $end_trip = TripPath::where('trip_id',$trip->id)->where('type','end_trip')->select('address')->first();
            if($pick_up)
            {
                $trip['pickup_address'] = $pick_up->address;
                if($trip->type == 'urgent')
                    $trip['endtrip_address'] = $end_trip->address;
                else
                    $trip['endtrip_address'] = $where_to->address;
            }
        }
        return $trips;
    }

    public function calculateTrip($input)
    {
//        $locations = $input->locations;
//
//        $total_distance = 0;
//        $waiting_time=0;
//        if (count($locations) >= 4 )
//        {
//            for ($i = 0, $j = 1; $j < count($locations)-2; $i+=2, $j+=2) {
//                $distanceKM =  getDistanceLatLng($locations[$i],$locations[$j],$locations[$i+2],$locations[$j+2]);
//                $total_distance += $distanceKM;
//                $distanceMeter = $distanceKM * 1000;
//                if($distanceMeter < 20)
//                    $waiting_time += 5;
//            }
//        }



        $locations = $input->locations;
//        $locations = array('30.12449,31.375885','30.12449,31.375885',
//                    '30.1243502,31.3757953','30.1236198,31.3749196','30.1232079,31.374411','30.1221937,31.3731624','30.1218785,31.3727314','30.1215097,31.3721974','30.1204943,31.3709169','30.1188341,31.3689759','30.1160976,31.3664356','30.1127029,31.3671662','30.1112891,31.3695268','30.1098672,31.3710297','30.1087037,31.372808','30.1079401,31.3744095','30.1068789,31.3748766','30.1062727,31.3741457','30.1059321,31.3735014','30.1052669,31.3727638','30.104587,31.3719825','30.1041925,31.3720628','30.1042933,31.3726417','30.1039022,31.3730937','30.10353,31.3735209','30.1033384,31.3737587','30.1033406,31.37376');;
                $waiting_time=0;
        $total_distance=0;

        $count=count($locations);
        $locations=json_encode($locations);
        if($count>0) {
//            $firstLastLocation = array($locations[0], $locations[1], $locations[$count - 2], $locations[$count - 1]);
            $actualTotal = getDistanceLatLng($locations);
//            $total = getDistanceLatLng($firstLastLocation);
//            if ($actualTotal['time'] >= $total['time']) {
//                $waiting_time = ($actualTotal['time'] - $total['time']) / 60;
//            }
            $total_distance = $actualTotal['distance'] ;
        }

        //store in DB

        $trip_distance = TripDistance::where('trip_id',$input->trip_id)->first();
        if($trip_distance)
        {
            $trip_distance->distance = $total_distance;
            $trip_distance->waiting_time = $waiting_time;
            $trip_distance->save();
        }else{
            TripDistance::create([
                'trip_id' => $input->trip_id,
                'distance' => $total_distance,
                'waiting_time' => $waiting_time
            ]);
        }
    }

    public function endTrip($input,$lang)
    {
        $trip = Trip::where('id',$input->trip_id)->first();
        $is_in_rush_period = $this->check_rush_time($trip);
         $this->calculateTrip($input);
//       dd($trip->car_id);
        if ($is_in_rush_period){
            //$service = RushHourService::where('country_id',$input->country_id)->first();
            $service = RushHourService::where('car_id',$trip->car_id)->first();
        }else{
            //$service = ServiceCar::where('country_id',$input->country_id)->first();
            $service = ServiceCar::where('car_id',$trip->car_id)->first();
        }

        if($service) {
           $car_price = Car::where('id',$trip->car_id)->first();

             $trip_distance = TripDistance::where('trip_id',$input->trip_id)->first();
           if($trip_distance)
           {
                   $total = $service->start + (($trip_distance->distance)
                      * $car_price->price) +
                  + $service->service_fees;
               //                  ($trip_distance->waiting_time * $service->wait)
               $promos_user = PromoUsed::where('user_id',$trip->user_id)->where('status',0)->get();
               if($promos_user)
               {
                   foreach ($promos_user as $promo_user)
                   {
                       $promo = Promo::where('id',$promo_user->promo_id)->
                       where('start_date','<=',Carbon::now())->where('end_date','>=',Carbon::now())->first();

                       if($promo)
                       {
                           if($total > $promo->amount)
                           {
                               $total -= $promo->amount;
                               $promo_user->update([
                                   'status' => 1
                               ]);
                           }
                       }
                   }
               }

           }else{
               $total = $service->minimum_fare;
           }

           $trip_total = number_format($total,2,'.', '');
           $locations=json_encode($input->locations);
           Trip::where('id',$input->trip_id)->update(['total' => $trip_total,'locations'=>$locations]);

           Driver::where('id',$trip->driver_id)->update(['status' => 0]);

           TripPath::create([
               'trip_id' => $input->trip_id,
               'type' => 'end_trip',
               'address' => $input->address,
               'lat' => $input->lat,
               'lng' => $input->lng
           ]);

           TripStatus::create([
              'trip_id' => $input->trip_id,
               'status_id' => 12
           ]);

           $ar_message = 'تم الانتهاء من الرحلة.';
           $en_message = 'Trip finished.';

//           UserNotify::create([
//               'user_id' => $trip->user_id,
//               'trip_id' => $input->trip_id,
//               'en' => ['name' => $en_message],
//               'ar' => ['name' => $ar_message]
//           ]);
           $user_notification = new UserNotify();
           $user_notification->user_id = $trip->user_id;
           $user_notification->trip_id = $input->trip_id;
           $user_notification->save();
           $user_notification->translateOrNew('ar')->name = $ar_message;
           $user_notification->translateOrNew('en')->name = $en_message;
           $user_notification->save();

           $driver = Driver::where('id',$trip->driver_id)->first();
           $driver_token = User::where('id', $trip->user_id)->pluck('token');
           Notify::send($driver_token, 'rate', $lang == 'ar' ? $ar_message : $en_message, '', $input->trip_id, $trip->driver_id,'','','','','',$driver->image,$driver->name,$trip_total);


           $percent = ($trip->total*14)/100;
           $driver->app_fees += number_format((float)$percent, 2, '.', '');
           $driver->save();

           return $trip_total;
       }
    }

    public function check_rush_time($trip)
    {
        $flag = false;
        if ($trip->type == 'urgent'){
            $trip_time = $trip->created_time;
        }else{
            $trip_time = $trip->time;
        }
        $rush_periods = RushHourPeriod::where('country_id', $trip->car->country->id)->get();
        foreach ($rush_periods as $rush_period){
//            dd($trip_time, $rush_period->to, $rush_period->from);
            if ($trip_time <= $rush_period->to && $trip_time >= $rush_period->from){
                $flag = true;
            }
        }
        return $flag;
    }

    public function collectMoney($input,$lang)
    {
        $trip = Trip::where('id',$input->trip_id)->first();

        if($trip) {

            $trip_total = number_format($trip->total,2,'.', '');
            $wallet = ($input->money) -  $trip_total;

            $user = User::whereId($trip->user_id)->first();

            $user->update(['wallet_amount' => ($user->wallet_amount) + $wallet]);

            $user->wallet_amount = ($user->wallet_amount) + $wallet;
            //
            $ar_message = 'تم اضافة '.$wallet. ' الي محفظتك.';
            $en_message = 'Your wallet has updated with'.$wallet;

//           UserNotify::create([
//               'user_id' => $trip->user_id,
//               'trip_id' => $input->trip_id,
//               'en' => ['name' => $en_message],
//               'ar' => ['name' => $ar_message]
//           ]);
            $user_notification = new UserNotify();
            $user_notification->user_id = $trip->user_id;
            $user_notification->trip_id = $input->trip_id;
            $user_notification->save();
            $user_notification->translateOrNew('ar')->name = $ar_message;
            $user_notification->translateOrNew('en')->name = $en_message;
            $user_notification->save();

            $driver = Driver::where('id',$trip->driver_id)->first();
            //$user_token = User::where('id', $trip->user_id)->pluck('token');
            Notify::send((array)$user->token,
                'rate',
                $lang == 'ar' ? $ar_message : $en_message,
                $lang == 'ar' ? $ar_message : $en_message,
                $input->trip_id,
                $trip->driver_id,
                '',
                $lang == 'ar' ? $ar_message : $en_message,
                '',
                '',
                '',
                $driver->image,
                '',
                $trip_total);

                TripStatus::create([
                    'trip_id' => $input->trip_id,
                    'status_id' => 14
                ]);
                           Trip::where('id',$input->trip_id)->update(['order_action' => 1]);


            return true;
        }
    }
    public function startTrip($input,$lang)
    {
        $trip = Trip::where('id',$input->trip_id)->first();
        if($trip) {
            $finished=TripStatus::where([
                'trip_id' => $input->trip_id,
                'status_id' => 8
            ])->first();

            if(!$finished) {


                TripStatus::create([
                    'trip_id' => $input->trip_id,
                    'status_id' => 8
                ]);
                $ar_message = 'بدأت الرحلة.';
                $en_message = 'Trip started.';

//           UserNotify::create([
//               'user_id' => $trip->user_id,
//               'trip_id' => $input->trip_id,
//               'en' => ['name' => $en_message],
//               'ar' => ['name' => $ar_message]
//           ]);
                $user_notification = new UserNotify();
                $user_notification->user_id = $trip->user_id;
                $user_notification->trip_id = $input->trip_id;
                $user_notification->save();
                $user_notification->translateOrNew('ar')->name = $ar_message;
                $user_notification->translateOrNew('en')->name = $en_message;
                $user_notification->save();

                $driver_token = User::where('id', $trip->user_id)->pluck('token');
                Notify::send($driver_token, 'start_trip', $lang == 'ar' ? $ar_message : $en_message, '', $input->trip_id);
            }
        }

    }
    public function nextTrip($input,$lang)
    {
        $trip = Trip::where('id',$input->trip_id)->first();
        //dd();
        if($trip) {
            $finished=TripStatus::where([
                'trip_id' => $input->trip_id,
                'status_id' => 12
            ])->first();

            if(!$finished) {


                TripStatus::create([
                    'trip_id' => $input->trip_id,
                    'status_id' => $trip->tripStatus->status_id+1
                ]);
                $ar_message = 'بدأ التحرك للموقع الأخر.';
                $en_message = 'start moving towards next location!.';

//           UserNotify::create([
//               'user_id' => $trip->user_id,
//               'trip_id' => $input->trip_id,
//               'en' => ['name' => $en_message],
//               'ar' => ['name' => $ar_message]
//           ]);
                $user_notification = new UserNotify();
                $user_notification->user_id = $trip->user_id;
                $user_notification->trip_id = $input->trip_id;
                $user_notification->save();
                $user_notification->translateOrNew('ar')->name = $ar_message;
                $user_notification->translateOrNew('en')->name = $en_message;
                $user_notification->save();

                $driver_token = User::where('id', $trip->user_id)->pluck('token');
                Notify::send($driver_token, 'start_trip', $lang == 'ar' ? $ar_message : $en_message, '', $input->trip_id);
            }
        }

    }

}
