<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;

class Trip extends Model
{
    protected $fillable = [
        'type','car_id','user_id','driver_id','order_action','canceled_by','rate','feedback','total','google_distance','date','time','locations'
    ];
    protected $appends = ['created_time'];

    public function tripPath()
    {
        return $this->hasMany(TripPath::class)->select('trip_id','type','address','lat','lng');
    }

    public function tripStatus()
    {
        return $this->hasOne(TripStatus::class)->select('id','trip_id','status_id')->latest('created_at');
    }

    public function user()
    {
        return $this->belongsTo(User::class)->select('id','name','phone','image','rate','payment_type');
    }
    public function car()
    {
        return $this->belongsTo(Car::class);
    }

    public function driver()
    {
        return $this->belongsTo(Driver::class)->select('id','name','phone','image','rate','car_id','country_id');
    }

    public function getCreatedAtAttribute()
    {
//        return Carbon::parse($this->attributes['created_at'])->format('d F Y');
        return Carbon::parse($this->attributes['created_at'])->toDateString();
    }

    public function getCreatedTimeAttribute()
    {
//        return Carbon::parse($this->attributes['created_at'])->format('d F Y');
        return Carbon::parse($this->attributes['created_at'])->format('H:i');
    }

    public function getRateAttribute()
    {
        return json_decode($this->attributes['rate'],true);
    }

    public function getFeedbackAttribute()
    {
        return json_decode($this->attributes['feedback'],true);
    }

    function distance($locations)
    {

//        $theta = $lon1 - $lon2;
//        $dist = sin(deg2rad($lat1)) * sin(deg2rad($lat2)) +  cos(deg2rad($lat1)) * cos(deg2rad($lat2)) * cos(deg2rad($theta));
//        $dist = acos($dist);
//        $dist = rad2deg($dist);
//        $miles = $dist * 60 * 1.1515;
//        $unit = strtoupper($unit);
//
//        if ($unit == "K") {
//            return ($miles * 1.609344);
//        } else if ($unit == "N") {
//            return ($miles * 0.8684);
//        } else {
//            return $miles;
//        }
//        $resultzzz=array('30.12449,31.375885','30.12449,31.375885',
//            '30.1243502,31.3757953','30.1236198,31.3749196','30.1232079,31.374411','30.1221937,31.3731624','30.1218785,31.3727314','30.1215097,31.3721974','30.1204943,31.3709169','30.1188341,31.3689759','30.1160976,31.3664356','30.1127029,31.3671662','30.1112891,31.3695268','30.1098672,31.3710297','30.1087037,31.372808','30.1079401,31.3744095','30.1068789,31.3748766','30.1062727,31.3741457','30.1059321,31.3735014','30.1052669,31.3727638','30.104587,31.3719825','30.1041925,31.3720628','30.1042933,31.3726417','30.1039022,31.3730937','30.10353,31.3735209','30.1033384,31.3737587','30.1033406,31.37376');




        //$indexer=intval($_GET['num1']);


    }

    public function search($search)
    {
        $builder = Trip::where('id', '>', 0);
        if ($search['search']){
//            $trips
            $builder->whereHas('user',function ($query) use($search){
                $query->where('name','like', '%'.$search['search'].'%')
                    ->orWhere('phone','like','%'.$search['search'].'%');
            })->orWhereHas('driver',function ($query) use($search){
                $query->where('name','like', '%'.$search['search'].'%')
                    ->orWhere('phone','like','%'.$search['search'].'%')
                    ->orWhereHas('driverCar',function ($q) use ($search){
                       $q->where('plat_number','like', '%'.$search['search'].'%');
                    });
            });
        }
        if($search['select_from'] && $search['select_to']) {
            $builder->where('created_at','>=',$search['select_from'])->where('created_at','<=',Carbon::parse($search['select_to'])->addDays(1));
        }

        if(isset($search['select_service_type'])) {
            $builder->where('order_action', $search['select_service_type']);
        }
        $trips = $builder->paginate(50);

        return $trips;

    }

    public function searchOld($search)
    {
        $trips = new Trip();

        $user = User::where(function($q) use($search)
        {
            $q->where('name','like','%'.$search['search'].'%');
        }
        )->first();

        $driver = Driver::where(function($q) use($search)
        {
            $q->where('name','like','%'.$search['search'].'%');
        }
        )->first();

        if($user)
        {
            $trips = $trips->where('user_id',$user->id);
        }

        if($driver)
        {
            $trips = $trips->where('driver_id',$driver->id);
        }

        if($search['select_from'] && $search['select_to'])
        {
            $trips = $trips->where('created_at','>=',$search['select_from'])->where('created_at','<=',Carbon::parse($search['select_to'])->addDays(1));
        }

        if($search['select_service_type'])
        {
            $trips = $trips->where('order_action', $search['select_service_type']);
        }

        $trips = $trips->latest()->paginate(50);

        return $trips;

    }
}
